/****************************************************************************
** Meta object code from reading C++ file 'videowidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/videowidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'videowidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QGraphicsViewExE_t {};
} // unnamed namespace

template <> constexpr inline auto QGraphicsViewEx::qt_create_metaobjectdata<qt_meta_tag_ZN15QGraphicsViewExE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGraphicsViewEx"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGraphicsViewEx, qt_meta_tag_ZN15QGraphicsViewExE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGraphicsViewEx::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QGraphicsViewExE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QGraphicsViewExE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QGraphicsViewExE_t>.metaTypes,
    nullptr
} };

void QGraphicsViewEx::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGraphicsViewEx *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QGraphicsViewEx::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGraphicsViewEx::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QGraphicsViewExE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int QGraphicsViewEx::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11videowidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto videowidget::qt_create_metaobjectdata<qt_meta_tag_ZN11videowidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "videowidget",
        "sigDeviceChange",
        "",
        "takePicCancel",
        "takePicDone",
        "takePicOnce",
        "takeVdCancel",
        "takeVdDone",
        "updateBlockSystem",
        "bTrue",
        "noCam",
        "noCamAvailable",
        "camAvailable",
        "filename",
        "strFilename",
        "switchCameraSuccess",
        "devName",
        "updateRecordState",
        "state",
        "updatePhotoState",
        "updateFilterImage",
        "QImage*",
        "image",
        "reflushSnapshotLabel",
        "onTakePic",
        "onTakeVideo",
        "showCountdown",
        "showRecTime",
        "onChangeDev",
        "onEndBtnClicked",
        "onRestartDevices",
        "onThemeTypeChanged",
        "DGuiApplicationHelper::ColorType",
        "themeType",
        "onExposureChanged",
        "exposure",
        "onFilterDisplayChanged",
        "bDisplay",
        "ReceiveMajorImage",
        "result",
        "ReceiveOpenGLstatus",
        "onReachMaxDelayedFrames",
        "flash",
        "slotresolutionchanged",
        "slotGridTypeChanged",
        "slotVideoFormatChanged",
        "onRecordFrame",
        "uchar*",
        "yuv",
        "size",
        "onRecordAudio",
        "data"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sigDeviceChange'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'takePicCancel'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'takePicDone'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'takePicOnce'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'takeVdCancel'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'takeVdDone'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateBlockSystem'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'noCam'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noCamAvailable'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'camAvailable'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filename'
        QtMocHelpers::SignalData<void(QString)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'switchCameraSuccess'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'updateRecordState'
        QtMocHelpers::SignalData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Signal 'updatePhotoState'
        QtMocHelpers::SignalData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Signal 'updateFilterImage'
        QtMocHelpers::SignalData<void(QImage *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'reflushSnapshotLabel'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTakePic'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'onTakeVideo'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showCountdown'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showRecTime'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onChangeDev'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onEndBtnClicked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRestartDevices'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onThemeTypeChanged'
        QtMocHelpers::SlotData<void(DGuiApplicationHelper::ColorType)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'onExposureChanged'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'onFilterDisplayChanged'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'ReceiveMajorImage'
        QtMocHelpers::SlotData<void(QImage *, int)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::Int, 39 },
        }}),
        // Slot 'ReceiveOpenGLstatus'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onReachMaxDelayedFrames'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'flash'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotresolutionchanged'
        QtMocHelpers::SlotData<void(const QString &)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'slotGridTypeChanged'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotVideoFormatChanged'
        QtMocHelpers::SlotData<void(const QString &)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'onRecordFrame'
        QtMocHelpers::SlotData<void(uchar *, uint)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 47, 48 }, { QMetaType::UInt, 49 },
        }}),
        // Slot 'onRecordAudio'
        QtMocHelpers::SlotData<void(uchar *, uint)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 47, 51 }, { QMetaType::UInt, 49 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<videowidget, qt_meta_tag_ZN11videowidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject videowidget::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11videowidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11videowidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11videowidgetE_t>.metaTypes,
    nullptr
} };

void videowidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<videowidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sigDeviceChange(); break;
        case 1: _t->takePicCancel(); break;
        case 2: _t->takePicDone(); break;
        case 3: _t->takePicOnce(); break;
        case 4: _t->takeVdCancel(); break;
        case 5: _t->takeVdDone(); break;
        case 6: _t->updateBlockSystem((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->noCam(); break;
        case 8: _t->noCamAvailable(); break;
        case 9: _t->camAvailable(); break;
        case 10: _t->filename((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->switchCameraSuccess((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->updateRecordState((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->updatePhotoState((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->updateFilterImage((*reinterpret_cast<std::add_pointer_t<QImage*>>(_a[1]))); break;
        case 15: _t->reflushSnapshotLabel(); break;
        case 16: _t->onTakePic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->onTakeVideo(); break;
        case 18: _t->showCountdown(); break;
        case 19: _t->showRecTime(); break;
        case 20: _t->onChangeDev(); break;
        case 21: _t->onEndBtnClicked(); break;
        case 22: _t->onRestartDevices(); break;
        case 23: _t->onThemeTypeChanged((*reinterpret_cast<std::add_pointer_t<DGuiApplicationHelper::ColorType>>(_a[1]))); break;
        case 24: _t->onExposureChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->onFilterDisplayChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->ReceiveMajorImage((*reinterpret_cast<std::add_pointer_t<QImage*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 27: _t->ReceiveOpenGLstatus((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->onReachMaxDelayedFrames(); break;
        case 29: _t->flash(); break;
        case 30: _t->slotresolutionchanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->slotGridTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->slotVideoFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: _t->onRecordFrame((*reinterpret_cast<std::add_pointer_t<uchar*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 34: _t->onRecordAudio((*reinterpret_cast<std::add_pointer_t<uchar*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::sigDeviceChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::takePicCancel, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::takePicDone, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::takePicOnce, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::takeVdCancel, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::takeVdDone, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)(bool )>(_a, &videowidget::updateBlockSystem, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::noCam, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::noCamAvailable, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::camAvailable, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)(QString )>(_a, &videowidget::filename, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)(const QString & )>(_a, &videowidget::switchCameraSuccess, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)(int )>(_a, &videowidget::updateRecordState, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)(int )>(_a, &videowidget::updatePhotoState, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)(QImage * )>(_a, &videowidget::updateFilterImage, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (videowidget::*)()>(_a, &videowidget::reflushSnapshotLabel, 15))
            return;
    }
}

const QMetaObject *videowidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *videowidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11videowidgetE_t>.strings))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int videowidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void videowidget::sigDeviceChange()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void videowidget::takePicCancel()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void videowidget::takePicDone()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void videowidget::takePicOnce()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void videowidget::takeVdCancel()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void videowidget::takeVdDone()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void videowidget::updateBlockSystem(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void videowidget::noCam()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void videowidget::noCamAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void videowidget::camAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void videowidget::filename(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void videowidget::switchCameraSuccess(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void videowidget::updateRecordState(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void videowidget::updatePhotoState(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void videowidget::updateFilterImage(QImage * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void videowidget::reflushSnapshotLabel()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
